@@ -58,13 +58,18 @@ that warning. (It’s generally better, of course, to handle all errors.)
 
 *is_in_progress = *is_pending;
 
+our $_NOTHING_CR = sub { };
+
 #----------------------------------------------------------------------
 # Undocumented, by design:
 
 sub set_deferral_AnyEvent() {
     require AnyEvent;
+
     ___set_deferral_generic(
-        \&AnyEvent::postpone,
+        AnyEvent->can('postpone') || \&_anyevent_postpone_compat,
+        undef,
+        _DEFER_ANYEVENT(),
     );
 }
 
@@ -72,17 +77,67 @@ sub set_deferral_IOAsync {
     my ($loop) = @_;
 
     ___set_deferral_generic(
-        $loop->can('later'),
+        $loop->can('later') || \&_ioasync_later_compat,
         $loop,
+        _DEFER_IOASYNC(),
+        sub { $loop->stop() },
     );
 }
 
 sub set_deferral_Mojo() {
     require Mojo::IOLoop;
+
     ___set_deferral_generic(
-        Mojo::IOLoop->can('next_tick'),
+        Mojo::IOLoop->can('next_tick') || \&_mojo_next_tick_compat,
         'Mojo::IOLoop',
+        _DEFER_MOJO(),
+        sub { Mojo::IOLoop->stop() },
     );
 }
 
+#----------------------------------------------------------------------
+# Polyfills for old versions of the event loops that didn’t include
+# zero-timer convenience functions:
+
+my %ae_timer;
+
+sub _anyevent_postpone_compat {
+    my $cb = $_[0];
+
+    my ($w, $w_str);
+
+    $w = AnyEvent->timer( after => 0, cb => sub {
+        delete $ae_timer{$w_str};
+        $cb->();
+    } );
+
+    $w_str = "$w";
+    $ae_timer{$w_str} = $w;
+}
+
+sub _ioasync_later_compat {
+    local ($@, $!);
+    require IO::Async::Timer;
+
+    $_[0]->add( IO::Async::Timer->new(
+        mode => 'countdown',
+        delay => 0,
+        on_expire => $_[1],
+    )->start() );
+}
+
+sub _mojo_next_tick_compat {
+    Mojo::IOLoop->timer( 0, $_[1] );
+}
+
 1;
+
+__END__
+
+sub _ioasync_wait_promise {
+    my $loop = $_[1];
+
+    $_[0]->catch(\&_NOTHING)->finally(sub { $loop->stop() } );
+
+    $loop->run();
+}
